//////////////////////////////////////////////////////
//	Lead-Lag Velocity Controller, tutorial version	//
//	by Jason Yang and Michael Hwang					//
//	Northwestern University, Summer 2007			//
//////////////////////////////////////////////////////

/*
These functions are intended to be used with the motor_pwm.h functions.
This closed loop lead-lag controller will take a speed in 
encoder counts per period as an input,and output a value between -100 and 100.
This output value should then be sent to the "setPDC" function 
in motor_pwm.h, which will adjust the speed of the motor.
*/

#ifndef __LEAD_LAG_H
#define __LEAD_LAG_H
#include "QEI.h"
#include <stdio.h>

/*
Choose whether you with to print the speed to the serial port by commenting or
uncommenting line
#define PRINT_SPEED
Beware: enabling printing the speed/position to USART takes up a lot of memory.
*/
#define PRINT_SPEED	//Comment out this line if you don't want to print the speed to USART

#define T 0.02		//period in seconds
#define K 0.8		//Gain
#define ZL 90.0		//lead zero
#define PL 100.0	//lead pole
#define Zl 50.0		//lag zero
#define Pl 20.0		//lag pole


int getLeadLag(signed int targetSpeed){

	static float sumError = 0;			
	static signed int  lastError = 0;
	static signed long lastQEIPos = 0;
	static signed long currentQEIPos; 	//must be signed long because of arithmetic.
	signed int currentSpeed;			//Must be signed int to avoid aliasing!
	signed int  currentError = 0;
	float diffError = 0;
	signed int outSignal;
	
	currentQEIPos = getQEIPos();
	currentSpeed = currentQEIPos - lastQEIPos;
	
	currentError = targetSpeed - currentSpeed;
	diffError = (currentError - lastError)/T;
	sumError = sumError+currentError*T;
	outSignal = K*((ZL+Zl)*currentError + ZL*Zl*sumError+diffError)/((PL+Pl)+(1/T)+(PL+Pl)*T);
	
	//Integrator anti-windup
	if (sumError > 10){
		sumError = 10;
	}
	else if (sumError < -10){
		sumError = -10;
	}
	
	//output saturation
	if (outSignal > 100){
		outSignal = 100;
	}
	else if (outSignal<-100){
		outSignal = -100;
	}
			
#if defined PRINT_SPEED
	printf("%i    \r", currentSpeed);
#endif

	lastError = currentError;
	lastQEIPos = currentQEIPos;	
	return outSignal;
}

#endif
